#!/usr/bin/perl

#
# Bluetooth Joke of the Day (BJOD) v0.1
#
# (c) Collin R. Mulliner <collin(AT)betaversion.net>
#
# web: www.mulliner.org/bluetooth/
#

# -- CONFIG --
 # send intro to unknown devices
 # 0 = no, 3 = yes
 $ADVERTISE = 3;
 # feed interval (in seconds)
 $INTERVAL = 60*60*24; # every 24h
 # BJOD idnet string
 $Cprefix = "#BJOD:";
 # name of ON command
 $Con = "on";
 # name of OFF command
 $Coff = "off";
 # name of HELP command
 $Chelp = "help";
 # seconds to search for devices
 $SearchTime = 10;
 # delay seconds between searches
 $DelayTime = 60;
 # file name that appears at the receiver
 $FdstfileName = "BJoD.txt";


# -- ADVANCED OPTIONS --

$debug = 0;

# -- prefs --
 # initial cookie for: getjoke.sh
 $COOKIE_INIT = 0;

# -- application path's --
$Phcitool = "/usr/bin/hcitool";
$Pussppush = "./ussp-push";
$Prfcomm = "/usr/bin/rfcomm";
$Psdptool = "/usr/bin/sdptool";
$Pgrep = "/bin/grep";
$Prm = "/bin/rm";

$Pgetjoke = "./getjoke.sh";

# -- bluetooth and rfcomm device --
$Drfcomm = "/dev/rfcomm0";
$Dhci = "hci0";

# -- config files --
 # file containing jokes (passed over to getjoke.sh)
 $Fjokes = "./jokes.bjod";
 # file containing the HELP text
 $Fhelp = "./help.bjod";
 # file containing the INTRODUCTION text
 $Fintro = "./intro.bjod";
 # device state storage
 $Fdevices = "./devices.bjod";

# ***********************************
#
# **  DON'T EDIT BELOW THIS LINE  **
#
# ***********************************

# --- CODE ---

sub dprint {
	if ($debug != 0) { print $_[0]; }
}

#
# @param 0 bd_addr
# @param 1 currentState
#
sub sendjoke {
	# sdptool get channel
	if ($DEVICES{$_[0]}[2] eq "") {
		dprint "getting obex channel\n";
		$channel = `$Psdptool search --bdaddr $_[0] OPUSH|grep Channel`;
		$channel =~ s/.* //gs;
		chomp($channel);
		if ($channel eq "") {
			return;
		}
		else {
			$DEVICES{$_[0]}[2] = $channel;
		}
	}
	dprint "channel: $channel\n";
	
	# bind rfcomm
	dprint "connecting...\n";
	@bla = `$Prfcomm -i $Dhci bind $Drfcomm $_[0] $DEVICES{$_[0]}[2]`;
	dprint "@bla";
	
	# send file
	$curjoke = $DEVICES{$_[0]}[1];
	
	if ($_[1] == 3) {
		dprint "sending: intro\n";
		@bla = `$Pussppush $Drfcomm $Fintro $FdstfileName`;
		# set state to off after intro
		$DEVICES{$_[0]}[0] = 0;
	}
	elsif ($_[1] == 2) {
		dprint "sending: help\n";
		@bla = `$Pussppush $Drfcomm $Fhelp $FdstfileName`;
	}
	else {
		$lasttime = $DEVICES{$_[0]}[3];
		$curtime = time();
		dprint "last: $lasttime, cur: $curtime, interval: $INTERVAL\n";
		if (($lasttime + $INTERVAL) <= $curtime) {
			dprint "sending: file\n";
			$joketmpfile = `$Pgetjoke $Fjokes $curjoke`;
			chomp($joketmpfile);
			dprint "cmd: $Pussppush $Drfcomm $joketmpfile $FdstfileName\n";
			@bla = `$Pussppush $Drfcomm $joketmpfile $FdstfileName`;
			
			# update cookie
			$DEVICES{$_[0]}[1] = $curjoke + 1;
			# update time
			$DEVICES{$_[0]}[3] = $curtime;
		}
		else {
			dprint "not sending because device is upto date!\n";
		}
	}
	dprint "@bla";
	
	# unbind rfcomm
	dprint "disconnecting...\n";
	@bla = `$Prfcomm release $Drfcomm`;
	dprint "@bla";
}

#
# save devices and settings
#
sub DEVICES_save {
	$bla = `$Prm -f $Fdevices`;
	open FP, "> $Fdevices";
	
	print FP "# BJOD - Devices file - don't edit!\n";
	while ( my ($key, @value) = each(%DEVICES) ) {
		print FP "$key;$DEVICES{$key}[0];$DEVICES{$key}[1];$DEVICES{$key}[2];$DEVICES{$key}[3]\n";
	}
	close FP;
}

#
# load devices and settings
#
sub DEVICES_load {
	open FP, "< $Fdevices";
	
	while (<FP>) {
		if ($_ =~ /^\#/) {
		}
		else {
			($lfmac, $lf1, $lf2, $lf3, $lf4) = split(/;/);
			chomp($lf4);
			dprint "loaded: BDADDR=$lfmac STATE=$lf1 COOKIE=$lf2 CHANNEL=$lf3 LAST_TIME=$lf4\n";
			$DEVICES{$lfmac}[0] = $lf1;
			$DEVICES{$lfmac}[1] = $lf2;
			$DEVICES{$lfmac}[2] = $lf3;
			$DEVICES{$lfmac}[3] = $lf4;
		}
	}
	close FP;
}

#
# main starts here
#


# first load old device settings
DEVICES_load();

while (1) {
	# scan for new devices
	@devs = `$Phcitool -i $Dhci scan --length=$SearchTime|$Pgrep -v Scanning`;
	dprint @devs;
	
	foreach my $curdev (@devs) {
		($garbage, $curdevMAC, $curdevNAME) = split(/\t/, $curdev);
		$curdevNAME =~ s/[\n\r]//;
		dprint "dev: $curdevMAC:\"$curdevNAME\"\n";
		
		if ($curdevNAME =~ /$Cprefix.*$/) {
			$curdevCMD = $curdevNAME;
			$curdevCMD =~ s/.*$Cprefix//gs;
			dprint "cmd: $curdevCMD\n";
		}
		else {
			$curdevCMD = "";
		}
		
		$curdevState = $DEVICES{$curdevMAC}[0];
		dprint "CHECK: >$curdevState<\n";
		
		if ($curdevState eq "") {
			dprint "new dev\n";
			$curdevState = $ADVERTISE;
			# initialize cookie
			$DEVICES{$curdevMAC}[1] = $COOKIE_INIT;
		}
		
		if ($curdevCMD =~ /$Con/) {
			$curdevState = 1;
			dprint "command on\n";
		}
		elsif ($curdevCMD =~ /$Coff/) {
			$curdevState = 0;
			dprint "command off\n";
		}
		elsif ($curdevCMD =~ /$Chelp/) {
			$curdevState = 2;
			dprint "command help\n";
		}

		# set new state but only if not help
		if ($curdevState != 2) {
			$DEVICES{$curdevMAC}[0] = $curdevState;
		}
		
		dprint "state: $curdevState\n";
		if ($curdevState != 0) {
			sendjoke($curdevMAC, $curdevState);
		}
	}
	
	# save DEVICES
	DEVICES_save();
	
	# wait before searching for more devices
	dprint "sleeping...\n";
	sleep $DelayTime;
	dprint "..............done\n";
}
